#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "structure.h"
#include "util.h"
#include "harv.h"

#define IN -99998

extern const char *REFINE_ID[];

extern PDBX_CRYSTAL_ALIGNMENT pdbx_crystal_alignment;
extern EXPTL_CRYSTAL exptl_crystal;
extern REFINE refine;
extern REFINE_NEUTRON refine_neutron;
extern REFINE_LS_SHELL refine_ls_shell;
extern REFINE_LS_SHELL refine_shell_neutron;
extern REFLNS_SHELL  reflns_shell ;
extern PDBX_REFINE pdbx_refine;
extern REFLNS  reflns ;
extern CELL cell ;
extern SYMMETRY symmetry;
extern COMPUTING computing;
extern PDBX_PHASING_DM  pdbx_phasing_dm;
extern PDBX_PHASING_DM_SHELL  pdbx_phasing_dm_shell;
extern PDBX_PHASING_MR  pdbx_phasing_MR;

extern PHASING_MAD phasing_mad;
extern PHASING_MAD phasing_mad;

extern PHASING_MIR_SHELL phasing_mir_shell;
extern EXPTL exptl;
extern AXIS axis;


void write_mmCIF(FILE *fout, int argc, char **argv)
{
    int i, index=0, scl=0,   dm=0, mr=0;
    
    for(i=1; i<argc; i++){
        if(!strcmp_case(argv[i], "-i")) index=1;
        else if (!strcmp_case(argv[i], "-s")) scl=1;
        else if (!strcmp_case(argv[i], "-sp")) scl=1;
        else if (!strcmp_case(argv[i], "-d")) dm=1;
        else if (!strcmp_case(argv[i], "-m")) mr=1;
//        else if (!strcmp_case(argv[i], "-e")) strcpy(method,argv[i+1]) ;
    }

    
    if(refine_neutron.ls_d_res_high>0 && refine.ls_d_res_high>0)  {
        fprintf(fout,"#\nloop_\n \
_exptl.entry_id\n \
_exptl.method \n \
_exptl.crystals_number\n \
");
        fprintf(fout,"%s  'X-RAY DIFFRACTION' %d \n ", DATA_BLOCK, exptl.crystals_number);
        fprintf(fout,"%s  'NEUTRON DIFFRACTION' %d \n ", DATA_BLOCK, exptl.crystals_number);
    }else {
        
        fprintf(fout,"#\n");
        fprintf(fout,"\n_exptl.crystals_number  %d\n", exptl.crystals_number);
        fprintf(fout,"_exptl.entry_id   %s\n", DATA_BLOCK);
        fprintf(fout,"_exptl.method   '%s'\n", exptl.method);
    }
    
    write_cell_symmetry(fout);
    if(scl<=0){
        write_reflns_shell(fout);
        write_reflns(fout);
    }
    
//    if(index>0) write_index(fout);
    
    write_exptl_crystal(fout);
    write_refine(fout);
    write_refine_ls_shell(fout);
    write_computing(fout);
    if(dm>0)write_dm(fout);
//    if(strstr_case(method, "MAD")) write_mad(fout);
    
    if(mr>0) write_mr(fout);
    
    
}

void write_mr(FILE *fout)
{
    
    fprintf(fout,"\n## Category = pdbx_phasing_MR\n");
    fprintf(fout,"\nloop_\n");
    fprintf(fout, "_pdbx_phasing_MR.entry_id\n");
    fprintf(fout, "_pdbx_phasing_MR.method_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.method_translation\n");
    fprintf(fout, "_pdbx_phasing_MR.model_details\n");
    fprintf(fout, "_pdbx_phasing_MR.R_factor\n");
    fprintf(fout, "_pdbx_phasing_MR.R_rigid_body\n");
    fprintf(fout, "_pdbx_phasing_MR.correlation_coeff_Fo_to_Fc\n");
    fprintf(fout, "_pdbx_phasing_MR.correlation_coeff_Io_to_Ic\n");
    fprintf(fout, "_pdbx_phasing_MR.d_res_high_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.d_res_low_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.d_res_high_translation\n");
    fprintf(fout, "_pdbx_phasing_MR.d_res_low_translation\n");
    fprintf(fout, "_pdbx_phasing_MR.d_res_high_fit\n");
    fprintf(fout, "_pdbx_phasing_MR.d_res_low_fit\n");
    fprintf(fout, "_pdbx_phasing_MR.packing\n");
    fprintf(fout, "_pdbx_phasing_MR.reflns_percent_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.reflns_percent_translation\n");
    fprintf(fout, "_pdbx_phasing_MR.sigma_F_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.sigma_F_translation\n");
    fprintf(fout, "_pdbx_phasing_MR.sigma_I_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.sigma_I_translation\n");
    fprintf(fout, "_pdbx_phasing_MR.zscore_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.LL_gain_rotation\n");
    fprintf(fout, "_pdbx_phasing_MR.zscore_translation\n");
    fprintf(fout, "_pdbx_phasing_MR.LL_gain_translation\n");
    
    fprintf(fout, "_pdbx_phasing_MR.angle_alpha\n");
    fprintf(fout, "_pdbx_phasing_MR.angle_beta\n");
    fprintf(fout, "_pdbx_phasing_MR.angle_gamma\n");
    fprintf(fout, "_pdbx_phasing_MR.Cartn_x\n");
    fprintf(fout, "_pdbx_phasing_MR.Cartn_y\n");
    fprintf(fout, "_pdbx_phasing_MR.Cartn_z\n");
    fprintf(fout, "_pdbx_phasing_MR.fract_x\n");
    fprintf(fout, "_pdbx_phasing_MR.fract_y\n");
    fprintf(fout, "_pdbx_phasing_MR.fract_z\n");

    fprintf(fout," \"%s\" ",DATA_BLOCK);
    (pdbx_phasing_MR.method_rotation[0]!='?') ? fprintf(fout," \"%s\" ",pdbx_phasing_MR.method_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.method_translation[0]!='?') ? fprintf(fout," \"%s\" ",pdbx_phasing_MR.method_translation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.model_details[0]!='?') ? fprintf(fout," \"%s\" ",pdbx_phasing_MR.model_details) : fprintf(fout," ? ");
    (pdbx_phasing_MR.R_factor >INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.R_factor) : fprintf(fout," ? ");
    (pdbx_phasing_MR.R_rigid_body>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.R_rigid_body) : fprintf(fout," ? ");
    (pdbx_phasing_MR.correlation_coeff_Fo_to_Fc>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.correlation_coeff_Fo_to_Fc) : fprintf(fout," ? ");
    (pdbx_phasing_MR.correlation_coeff_Io_to_Ic>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.correlation_coeff_Io_to_Ic) : fprintf(fout," ? ");
    (pdbx_phasing_MR.d_res_high_rotation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.d_res_high_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.d_res_low_rotation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.d_res_low_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.d_res_high_translation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.d_res_high_translation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.d_res_low_translation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.d_res_low_translation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.d_res_high_fit>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.d_res_high_fit) : fprintf(fout," ? ");
    (pdbx_phasing_MR.d_res_low_fit>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.d_res_low_fit) : fprintf(fout," ? ");
    (pdbx_phasing_MR.packing>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.packing) : fprintf(fout," ? ");
    (pdbx_phasing_MR.reflns_percent_rotation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.reflns_percent_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.reflns_percent_translation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.reflns_percent_translation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.sigma_F_rotation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.sigma_F_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.sigma_F_translation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.sigma_F_translation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.sigma_I_rotation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.sigma_I_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.sigma_I_translation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.sigma_I_translation) : fprintf(fout," ? ");
    
    (pdbx_phasing_MR.zscore_rotation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.zscore_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.LL_gain_rotation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.LL_gain_rotation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.zscore_translation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.zscore_translation) : fprintf(fout," ? ");
    (pdbx_phasing_MR.LL_gain_translation>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.LL_gain_translation) : fprintf(fout," ? ");



    (pdbx_phasing_MR.angle_alpha>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.angle_alpha) : fprintf(fout," ? ");
    (pdbx_phasing_MR.angle_beta>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.angle_beta) : fprintf(fout," ? ");
    (pdbx_phasing_MR.angle_gamma>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.angle_gamma) : fprintf(fout," ? ");
    (pdbx_phasing_MR.Cartn_x>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.Cartn_x) : fprintf(fout," ? ");
    (pdbx_phasing_MR.Cartn_y>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.Cartn_y) : fprintf(fout," ? ");
    (pdbx_phasing_MR.Cartn_z>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.Cartn_z) : fprintf(fout," ? ");
    (pdbx_phasing_MR.fract_x>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.fract_x) : fprintf(fout," ? ");
    (pdbx_phasing_MR.fract_y>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.fract_y) : fprintf(fout," ? ");
    (pdbx_phasing_MR.fract_z>INI) ? fprintf(fout," %.3f ",pdbx_phasing_MR.fract_z) : fprintf(fout," ? ");
    
    fprintf(fout, "\n");
}

void write_dm(FILE *fout)
{
    int i;

    
    fprintf(fout,"_pdbx_phasing_dm.entry_id \"%s\"\n", DATA_BLOCK);
    
    if(pdbx_phasing_dm.mask_type[0] != '?')
        fprintf(fout,"_pdbx_phasing_dm.mask_type \"%s\"\n", pdbx_phasing_dm.mask_type);
    
    if(pdbx_phasing_dm.method[0] != '?')
        fprintf(fout,"_pdbx_phasing_dm.method \"%s\"\n", pdbx_phasing_dm.method);

    
    if(pdbx_phasing_dm.delta_phi_final>INI)
        fprintf(fout,"_pdbx_phasing_dm.delta_phi_final %.3f\n", pdbx_phasing_dm.delta_phi_final);
    
    if(pdbx_phasing_dm.delta_phi_initial>INI)
        fprintf(fout,"_pdbx_phasing_dm.delta_phi_initial %.3f\n", pdbx_phasing_dm.delta_phi_initial);
    
    if(pdbx_phasing_dm.fom_acentric>INI)
        fprintf(fout,"_pdbx_phasing_dm.fom_acentric %.3f\n", pdbx_phasing_dm.fom_acentric);
    
    if(pdbx_phasing_dm.fom_centric>INI)
        fprintf(fout,"_pdbx_phasing_dm.fom_centric %.3f\n", pdbx_phasing_dm.fom_centric);
    
    if(pdbx_phasing_dm.fom>INI)
        fprintf(fout,"_pdbx_phasing_dm.fom %.3f\n", pdbx_phasing_dm.fom);
    
    if(pdbx_phasing_dm.reflns_acentric>INI)
        fprintf(fout,"_pdbx_phasing_dm.reflns_acentric %d\n", pdbx_phasing_dm.reflns_acentric);
    
    if(pdbx_phasing_dm.reflns_centric>INI)
        fprintf(fout,"_pdbx_phasing_dm.reflns_centric %d\n", pdbx_phasing_dm.reflns_centric);
    
    if(pdbx_phasing_dm.reflns>INI)
        fprintf(fout,"_pdbx_phasing_dm.reflns %d\n", pdbx_phasing_dm.reflns);
    
    if(pdbx_phasing_dm.mask_radius>INI)
        fprintf(fout,"_pdbx_phasing_dm.mask_radius %.3f\n", pdbx_phasing_dm.mask_radius);
    if(pdbx_phasing_dm.d_res_high>INI)
        fprintf(fout,"_pdbx_phasing_dm.d_res_high %.3f\n", pdbx_phasing_dm.d_res_high);
    
    if(pdbx_phasing_dm.d_res_low>INI)
        fprintf(fout,"_pdbx_phasing_dm.d_res_low %.3f\n", pdbx_phasing_dm.d_res_low);
    if(pdbx_phasing_dm.protein_percent_sol>INI)
        fprintf(fout,"_pdbx_phasing_dm.protein_percent_sol %.3f\n", pdbx_phasing_dm.protein_percent_sol);
    
    if(pdbx_phasing_dm.protein_min_density>INI)
        fprintf(fout,"_pdbx_phasing_dm.protein_min_density %.3f\n", pdbx_phasing_dm.protein_min_density);
    if(pdbx_phasing_dm.protein_max_density>INI)
        fprintf(fout,"_pdbx_phasing_dm.protein_max_density %.3f\n", pdbx_phasing_dm.protein_max_density);
    
    if(pdbx_phasing_dm.protein_mean_density>INI)
        fprintf(fout,"_pdbx_phasing_dm.protein_mean_density %.3f\n", pdbx_phasing_dm.protein_mean_density);
    if(pdbx_phasing_dm.protein_mean_density_esd>INI)
        fprintf(fout,"_pdbx_phasing_dm.protein_mean_density_esd %.3f\n", pdbx_phasing_dm.protein_mean_density_esd);
    
    if(pdbx_phasing_dm.solvent_percent_sol>INI)
        fprintf(fout,"_pdbx_phasing_dm.solvent_percent_sol %.3f\n", pdbx_phasing_dm.solvent_percent_sol);
    if(pdbx_phasing_dm.solvent_min_density>INI)
        fprintf(fout,"_pdbx_phasing_dm.solvent_min_density %.3f\n", pdbx_phasing_dm.solvent_min_density);
    
    if(pdbx_phasing_dm.solvent_max_density>INI)
        fprintf(fout,"_pdbx_phasing_dm.solvent_max_density %.3f\n", pdbx_phasing_dm.solvent_max_density);
    if(pdbx_phasing_dm.solvent_mean_density>INI)
        fprintf(fout,"_pdbx_phasing_dm.solvent_mean_density %.3f\n", pdbx_phasing_dm.solvent_mean_density);
    
    if(pdbx_phasing_dm.solvent_mean_density_esd>INI)
        fprintf(fout,"_pdbx_phasing_dm.solvent_mean_density_esd %.3f\n", pdbx_phasing_dm.solvent_mean_density_esd);

/*    
    if(pdbx_phasing_dm.prior_map_CC_acentric>INI)
        fprintf(fout,"_pdbx_phasing_dm.prior_map_CC_acentric %.3f\n", pdbx_phasing_dm.prior_map_CC_acentric);

    if(pdbx_phasing_dm.prior_map_CC_centric>INI)
        fprintf(fout,"_pdbx_phasing_dm.prior_map_CC_centric %.3f\n", pdbx_phasing_dm.prior_map_CC_centric);

    if(pdbx_phasing_dm.prior_map_CC>INI)
        fprintf(fout,"_pdbx_phasing_dm.prior_map_CC %.3f\n", pdbx_phasing_dm.prior_map_CC);
*/
    
    if(pdbx_phasing_dm_shell.number_bin>=1){
        fprintf(fout,"\n\n##### Density modification #####\n");
        fprintf(fout,"loop_\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.d_res_high\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.d_res_low\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.delta_phi_final\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.delta_phi_initial\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.fom_acentric\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.fom_centric\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.fom\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.reflns_acentric\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.reflns_centric\n");
        fprintf(fout,"_pdbx_phasing_dm_shell.reflns\n");
        
//        fprintf(fout,"_pdbx_phasing_dm_shell.prior_map_CC_acentric\n");
//        fprintf(fout,"_pdbx_phasing_dm_shell.prior_map_CC_centric\n");
//        fprintf(fout,"_pdbx_phasing_dm_shell.prior_map_CC\n");
        for(i=0; i<pdbx_phasing_dm_shell.number_bin; i++){
            (pdbx_phasing_dm_shell.d_res_high[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.d_res_high[i]) : fprintf(fout," ?\n");
         
            (pdbx_phasing_dm_shell.d_res_low[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.d_res_low[i]) : fprintf(fout," ? ");
         
            (pdbx_phasing_dm_shell.delta_phi_final[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.delta_phi_final[i]) : fprintf(fout," ? ");
         
            (pdbx_phasing_dm_shell.delta_phi_initial[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.delta_phi_initial[i]) : fprintf(fout," ? ");
         
            (pdbx_phasing_dm_shell.fom_acentric[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.fom_acentric[i]) : fprintf(fout," ? ");
         
            (pdbx_phasing_dm_shell.fom_centric[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.fom_centric[i]) : fprintf(fout," ? ");
         
            (pdbx_phasing_dm_shell.fom[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.fom[i]) : fprintf(fout," ? ");
         
            (pdbx_phasing_dm_shell.reflns_acentric[i]>0)?
                fprintf(fout," %d ", pdbx_phasing_dm_shell.reflns_acentric[i]) : fprintf(fout," ? ");

            (pdbx_phasing_dm_shell.reflns_centric[i]>0)?
                fprintf(fout," %d ", pdbx_phasing_dm_shell.reflns_centric[i]) : fprintf(fout," ? ");
         
            (pdbx_phasing_dm_shell.reflns[i]>0)?
                fprintf(fout," %d ", pdbx_phasing_dm_shell.reflns[i]) : fprintf(fout," ? ");
/*
            
            (pdbx_phasing_dm_shell.prior_map_CC_acentric[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.prior_map_CC_acentric[i]) : fprintf(fout," ? ");
            
            (pdbx_phasing_dm_shell.prior_map_CC_centric[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.prior_map_CC_centric[i]) : fprintf(fout," ? ");
            
            (pdbx_phasing_dm_shell.prior_map_CC[i]>0)?
                fprintf(fout," %.3f ", pdbx_phasing_dm_shell.prior_map_CC[i]) : fprintf(fout," ? ");
*/            
            fprintf(fout,"\n");
            
        }
    }
    

}
void write_mad(FILE *fout)
{
 //_phasing_mad
    if(phasing_mad.pdbx_d_res_high && phasing_mad.pdbx_d_res_low){
        fprintf(fout, "_phasing_MAD.entry_id  '%s'\n",DATA_BLOCK);
        fprintf(fout, "_phasing_mad.pdbx_d_res_high  '%.2f'\n", phasing_mad.pdbx_d_res_high);
    
        fprintf(fout, "_phasing_mad.pdbx_d_res_low  '%.2f'\n", phasing_mad.pdbx_d_res_low);
        
    }
    
}


void write_cell_symmetry(FILE *fout)
{
    char sg[20];
    
 //cell
    if(cell.length_a>0) {
        fprintf(fout,"#\n");
    
        fprintf(fout,"_cell.entry_id \"%s\"\n", DATA_BLOCK);
        fprintf(fout,"_cell.length_a ");
        (cell.length_a>0) ? fprintf(fout," %8.3f\n", cell.length_a) : fprintf(fout,"?\n");
    
        fprintf(fout,"_cell.length_b ");
        (cell.length_b>0) ? fprintf(fout," %8.3f\n", cell.length_b) : fprintf(fout,"?\n");

        fprintf(fout,"_cell.length_c ");
        (cell.length_c>0) ? fprintf(fout," %8.3f\n", cell.length_c) : fprintf(fout,"?\n");

        fprintf(fout,"_cell.angle_alpha  ");
        (cell.angle_alpha>0) ? fprintf(fout," %8.3f\n", cell.angle_alpha ) : fprintf(fout,"?\n");

        fprintf(fout,"_cell.angle_beta  ");
        (cell.angle_beta>0) ? fprintf(fout," %8.3f\n", cell.angle_beta ) : fprintf(fout,"?\n");

        fprintf(fout,"_cell.angle_gamma   ");
        (cell.angle_gamma >0) ? fprintf(fout," %8.3f\n", cell.angle_gamma  ) : fprintf(fout,"?\n");

        if(cell.length_a_esd>=0) fprintf(fout,"_cell.length_a_esd %.3f\n ", cell.length_a_esd);
        if(cell.length_b_esd>=0) fprintf(fout,"_cell.length_b_esd %.3f\n ", cell.length_b_esd);
        if(cell.length_c_esd>=0) fprintf(fout,"_cell.length_c_esd %.3f\n ", cell.length_c_esd);
        if(cell.angle_alpha_esd>=0) fprintf(fout,"_cell.angle_alpha_esd %.3f\n ", cell.angle_alpha_esd);
        if(cell.angle_beta_esd>=0) fprintf(fout,"_cell.angle_beta_esd %.3f\n ", cell.angle_beta_esd);
        if(cell.angle_gamma_esd>=0) fprintf(fout,"_cell.angle_gamma_esd %.3f\n ", cell.angle_gamma_esd);
    }

    
// symmetry    
    if(symmetry.Int_Tables_number>0 || strlen(symmetry.space_group_name_H_M)>1){
        
        fprintf(fout,"#\n");
        
        fprintf(fout,"_symmetry.entry_id %s\n", DATA_BLOCK);
        if (symmetry.Int_Tables_number>0) 
            fprintf(fout,"_symmetry.Int_Tables_number  %d\n", symmetry.Int_Tables_number);
        space_group_convertor(symmetry.space_group_name_H_M, sg);
        
        fprintf(fout,"_symmetry.space_group_name_H-M  '%s' \n", sg);
    }
    
    
}


void write_computing(FILE *fout)
//.data_collection changed to pdbx_data_reduction_ds
//.data_collection changed to pdbx_data_reduction_ii  
{
    
    fprintf(fout,"_computing.entry_id  \"%s\" \n", DATA_BLOCK);
    fprintf(fout,"_computing.structure_refinement  \"%s\" \n", computing.structure_refinement);
    fprintf(fout,"_computing.structure_solution  \"%s\" \n", computing.structure_solution);
//    fprintf(fout,"_computing.pdbx_density_modification  \"%s\" \n", computing.pdbx_density_modification);
    fprintf(fout,"_computing.pdbx_data_reduction_ds \"%s\" \n", computing.data_reduction);
    fprintf(fout,"_computing.pdbx_data_reduction_ii  \"%s\" \n", computing.data_collection);
    fprintf(fout,"_computing.cell_refinement  \"%s\" \n", computing.cell_refinement);
}


void write_index(FILE *fout)
{
    const char *items[19] =
        {
            "crystal_id",
            "xbeam",
            "xbeam_esd",
            "ybeam",
            "ybeam_esd",
            "crysx_spindle",
            "crysx_spindle_esd",
            "crysy_vertical",
            "crysy_vertical_esd",
            "crysz_beam",
            "crysz_beam_esd",
            "crystal_to_detector_distance",
            "crystal_to_detector_distance_esd",
            "crossfire_y",
            "crossfire_y_esd",
            "crossfire_x",
            "crossfire_x_esd",
            "crossfire_xy",
            "crossfire_xy_esd"
        };
    print_mmcif_head(fout,"_pdbx_crystal_alignment", items, 19);
    (pdbx_crystal_alignment.crystal_id>IN) ?fprintf(fout," %d ",pdbx_crystal_alignment.crystal_id): fprintf(fout," 1 ");
    (pdbx_crystal_alignment.xbeam>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.xbeam): fprintf(fout," ? ");
    (pdbx_crystal_alignment.xbeam_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.xbeam_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.ybeam>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.ybeam): fprintf(fout," ? ");
    (pdbx_crystal_alignment.ybeam_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.ybeam_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crysx_spindle>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crysx_spindle): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crysx_spindle_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crysx_spindle_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crysy_vertical>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crysy_vertical): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crysy_vertical_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crysy_vertical_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crysz_beam>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crysz_beam): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crysz_beam_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crysz_beam_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crystal_to_detector_distance>IN) ?
        fprintf(fout," %.3f ",pdbx_crystal_alignment.crystal_to_detector_distance): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crystal_to_detector_distance_esd>IN) ?
        fprintf(fout," %.3f ",pdbx_crystal_alignment.crystal_to_detector_distance_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crossfire_y>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crossfire_y): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crossfire_y_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crossfire_y_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crossfire_x>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crossfire_x): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crossfire_x_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crossfire_x_esd): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crossfire_xy>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crossfire_xy): fprintf(fout," ? ");
    (pdbx_crystal_alignment.crossfire_xy_esd>IN) ?fprintf(fout," %.3f ",pdbx_crystal_alignment.crossfire_xy_esd): fprintf(fout," ? ");

    fprintf(fout,"\n");

    if(axis.key>0){
        const char *item[6] =
            {
                "id",
                "type",
                "equipment",
                "vector[1]",
                "vector[2]",
                "vector[3]",
            };
        print_mmcif_head(fout,"_axis", item, 6);
        fprintf(fout,"omega rotation goniometer %.5f %.5f %.5f \n", axis.omega[0], axis.omega[1], axis.omega[2]);
        fprintf(fout,"kappa rotation goniometer %.5f %.5f %.5f \n", axis.kappa[0], axis.kappa[1], axis.kappa[2]);
        fprintf(fout,"phi   rotation goniometer %.5f %.5f %.5f \n", axis.phi[0], axis.phi[1], axis.phi[2]);
    }
    
        
        
        
}


            
    
void write_exptl_crystal(FILE *fout)
{
    const char *items[] =
        {
            "id",
            "pdbx_mosaicity",
            "pdbx_mosaicity_esd",
            "density_Matthews",
            "density_diffrn",
            "density_meas",
            "density_meas_temp",
            "density_percent_sol",
            "size_max",
            "size_mid",
            "size_min",
            "size_rad"
        };
    
    print_mmcif_head(fout,"_exptl_crystal", items, 12);
    
    (exptl_crystal.id>INI)? fprintf(fout," %d ",exptl_crystal.id):fprintf(fout," 1 ");
    (exptl_crystal.pdbx_mosaicity>INI)? fprintf(fout," %.3f  ",exptl_crystal.pdbx_mosaicity):fprintf(fout," ? ");
    (exptl_crystal.pdbx_mosaicity_esd>INI)? fprintf(fout," %.3f ",exptl_crystal.pdbx_mosaicity_esd):fprintf(fout," ? ");
    (exptl_crystal.density_Matthews>INI)? fprintf(fout," %.3f ",exptl_crystal.density_Matthews):fprintf(fout," ? ");
    (exptl_crystal.density_diffrn>INI)? fprintf(fout," %.3f ",exptl_crystal.density_diffrn):fprintf(fout," ? ");
    (exptl_crystal.density_meas>INI)? fprintf(fout," %.3f ",exptl_crystal.density_meas):fprintf(fout," ? ");
    (exptl_crystal.density_meas_temp>INI)? fprintf(fout," %.3f ",exptl_crystal.density_meas_temp):fprintf(fout," ? ");
    (exptl_crystal.density_percent_sol>INI)? fprintf(fout," %.3f ",exptl_crystal.density_percent_sol):fprintf(fout," ? ");
    (exptl_crystal.size_max>INI)? fprintf(fout," %.3f ",exptl_crystal.size_max):fprintf(fout," ? ");
    (exptl_crystal.size_mid>INI)? fprintf(fout," %.3f ",exptl_crystal.size_mid):fprintf(fout," ? ");
    (exptl_crystal.size_min>INI)? fprintf(fout," %.3f ",exptl_crystal.size_min):fprintf(fout," ? ");
    (exptl_crystal.size_rad>INI)? fprintf(fout," %.3f ",exptl_crystal.size_rad):fprintf(fout," ? ");
    
    fprintf(fout,"\n");
    
}
    
void write_refine(FILE *fout)
{
    int ini=-9998;

    
    if(refine.ls_number_reflns_R_work>0 && refine.ls_number_reflns_R_free >0 &&
      refine.ls_number_reflns_obs <=0){
        refine.ls_number_reflns_obs = refine.ls_number_reflns_R_work + refine.ls_number_reflns_R_free;
        if(reflns.number_obs > 0 && refine.ls_number_reflns_obs >reflns.number_obs )
            refine.ls_number_reflns_obs = refine.ls_number_reflns_R_work;
    }

    fprintf(fout,"#\n");
    fprintf(fout,"loop_ \n \
_refine.entry_id \n \
_refine.pdbx_refine_id \n \
_refine.ls_d_res_high \n \
_refine.ls_d_res_low \n \
_refine.pdbx_ls_sigma_F \n \
_refine.pdbx_data_cutoff_high_absF \n \
_refine.pdbx_data_cutoff_low_absF \n \
_refine.ls_percent_reflns_obs \n \
_refine.ls_number_reflns_obs \n \
_refine.ls_number_reflns_all \n \
_refine.pdbx_ls_cross_valid_method \n \
_refine.ls_matrix_type \n \
_refine.pdbx_R_Free_selection_details \n \
_refine.details \n \
_refine.ls_R_factor_all \n \
_refine.ls_R_factor_obs \n \
_refine.ls_R_factor_R_work \n \
_refine.ls_wR_factor_R_work \n \
_refine.ls_R_factor_R_free \n \
_refine.ls_wR_factor_R_free \n \
_refine.ls_percent_reflns_R_free \n \
_refine.ls_number_reflns_R_free \n \
_refine.ls_number_reflns_R_work \n \
_refine.ls_R_factor_R_free_error \n \
_refine.B_iso_mean \n \
_refine.solvent_model_param_bsol \n \
_refine.solvent_model_param_ksol \n \
_refine.pdbx_isotropic_thermal_model  \n \
_refine.aniso_B[1][1] \n \
_refine.aniso_B[2][2] \n \
_refine.aniso_B[3][3] \n \
_refine.aniso_B[1][2] \n \
_refine.aniso_B[1][3]  \n \
_refine.aniso_B[2][3] \n \
_refine.correlation_coeff_Fo_to_Fc \n \
_refine.correlation_coeff_Fo_to_Fc_free \n \
_refine.overall_SU_R_Cruickshank_DPI \n \
_refine.overall_SU_R_free \n \
_refine.pdbx_overall_ESU_R  \n \
_refine.pdbx_overall_ESU_R_Free \n \
_refine.overall_SU_ML \n \
_refine.overall_SU_B \n \
_refine.solvent_model_details  \n \
_refine.pdbx_solvent_vdw_probe_radii  \n \
_refine.pdbx_solvent_ion_probe_radii \n \
_refine.pdbx_solvent_shrinkage_radii  \n \
_refine.ls_number_parameters  \n \
_refine.ls_number_restraints \n \
_refine.pdbx_starting_model \n \
_refine.pdbx_method_to_determine_struct \n \
_refine.pdbx_stereochemistry_target_values \n \
_refine.pdbx_stereochem_target_val_spec_case \n \
_refine.overall_FOM_work_R_set \n \
_refine.B_iso_max \n \
_refine.B_iso_min \n \
_refine.pdbx_overall_phase_error \n \
_refine.occupancy_max  \n \
_refine.occupancy_min \n\n ");

    fprintf(fout,"%s  '%s' ",  DATA_BLOCK, REFINE_ID[0]);
    
    (refine.ls_d_res_high >ini)? fprintf(fout,"  %.4f ", refine.ls_d_res_high):fprintf(fout," ? ");

    (refine.ls_d_res_low >ini)? fprintf(fout,"  %.4f ", refine.ls_d_res_low):fprintf(fout," ? ");
   
   (refine.pdbx_ls_sigma_F  >ini)? fprintf(fout,"   %.3f ", refine.pdbx_ls_sigma_F):fprintf(fout," ? ");
   
   (refine.pdbx_data_cutoff_high_absF >ini)? fprintf(fout,"  %.4f ", refine.pdbx_data_cutoff_high_absF):fprintf(fout," ? ");
   
   (refine.pdbx_data_cutoff_low_absF >ini)? fprintf(fout,"  %.4f ", refine.pdbx_data_cutoff_low_absF):fprintf(fout," ? ");
   
   (refine.ls_percent_reflns_obs >ini)? fprintf(fout,"  %.4f ", refine.ls_percent_reflns_obs):fprintf(fout," ? ");
   
   (refine.ls_number_reflns_obs >ini)? fprintf(fout,"  %d ", refine.ls_number_reflns_obs):fprintf(fout," ? ");

   (refine.ls_number_reflns_all >ini)? fprintf(fout,"  %d ", refine.ls_number_reflns_all):fprintf(fout," ? ");
   
   (refine.pdbx_ls_cross_valid_method[0] != '?')? fprintf(fout," \"%s\"  ", refine.pdbx_ls_cross_valid_method):fprintf(fout," ? ");
   
   (refine.ls_matrix_type[0] != '?') ?fprintf(fout," \"%s\"   ", refine.ls_matrix_type):fprintf(fout," ? ");
   
   (refine.pdbx_R_Free_selection_details[0] !='?') ?fprintf(fout," \"%s\"   ", refine.pdbx_R_Free_selection_details):fprintf(fout," ? ");

   (refine.details[0] !='?' && strlen(refine.details) >2)? fprintf(fout," \"%s\"  ", refine.details):fprintf(fout," ? ");
   
   (refine.ls_R_factor_all >ini)?  fprintf(fout,"  %.4f ", refine.ls_R_factor_all):fprintf(fout," ? ");
   
   (refine.ls_R_factor_obs >ini)? fprintf(fout,"  %.4f ", refine.ls_R_factor_obs):fprintf(fout," ? ");
   
   (refine.ls_R_factor_R_work >ini)? fprintf(fout,"  %.4f ", refine.ls_R_factor_R_work):fprintf(fout," ? ");
   
   (refine.ls_wR_factor_R_work >ini)? fprintf(fout,"  %.4f ", refine.ls_wR_factor_R_work):fprintf(fout," ? ");
   
   (refine.ls_R_factor_R_free >ini)? fprintf(fout,"  %.4f ", refine.ls_R_factor_R_free):fprintf(fout," ? ");
   
   (refine.ls_wR_factor_R_free >ini)? fprintf(fout,"  %.4f ", refine.ls_wR_factor_R_free):fprintf(fout," ? ");
   
   (refine.ls_percent_reflns_R_free >ini)? fprintf(fout,"  %.4f ", refine.ls_percent_reflns_R_free):fprintf(fout," ? ");
   
   (refine.ls_number_reflns_R_free >ini)? fprintf(fout,"  %d ", refine.ls_number_reflns_R_free):fprintf(fout," ? ");

   (refine.ls_number_reflns_R_work >0)? fprintf(fout,"  %d ", refine.ls_number_reflns_R_work):fprintf(fout," ? ");
   
   (refine.ls_R_factor_R_free_error >ini)? fprintf(fout,"  %.4f ", refine.ls_R_factor_R_free_error):fprintf(fout," ? ");

   (refine.B_iso_mean >ini)?  fprintf(fout,"  %.4f ", refine.B_iso_mean):fprintf(fout," ? ");

   (refine.solvent_model_param_bsol >ini)? fprintf(fout,"  %.4f ", refine.solvent_model_param_bsol):fprintf(fout," ? ");

   (refine.solvent_model_param_ksol >ini)? fprintf(fout," %.4f ", refine.solvent_model_param_ksol):fprintf(fout," ? ");

   (refine.pdbx_isotropic_thermal_model[0] != '?' )? fprintf(fout," \"%s\"  ", refine.pdbx_isotropic_thermal_model):fprintf(fout," ? ");

   (refine.aniso_B11 >ini)? fprintf(fout,"  %.4f ", refine.aniso_B11):fprintf(fout," ? ");

   (refine.aniso_B22 >ini)? fprintf(fout,"  %.4f ", refine.aniso_B22):fprintf(fout," ? ");

   (refine.aniso_B33 >ini)?  fprintf(fout,"  %.4f ", refine.aniso_B33):fprintf(fout," ? ");

   (refine.aniso_B12 >ini)? fprintf(fout,"  %.4f ", refine.aniso_B12):fprintf(fout," ? ");

   (refine.aniso_B13 >ini)? fprintf(fout," %.4f ", refine.aniso_B13):fprintf(fout," ? ");

   (refine.aniso_B23 >ini)? fprintf(fout,"  %.4f ", refine.aniso_B23):fprintf(fout," ? ");

   (refine.correlation_coeff_Fo_to_Fc >ini)? fprintf(fout,"  %.4f ", refine.correlation_coeff_Fo_to_Fc):fprintf(fout," ? ");

   (refine.correlation_coeff_Fo_to_Fc_free >ini)?fprintf(fout,"  %.4f ", refine.correlation_coeff_Fo_to_Fc_free):fprintf(fout," ? ");

   (refine.overall_SU_R_Cruickshank_DPI >ini)? fprintf(fout,"  %.4f ", refine.overall_SU_R_Cruickshank_DPI):fprintf(fout," ? ");

   (refine.overall_SU_R_free >ini)? fprintf(fout,"  %.4f ", refine.overall_SU_R_free):fprintf(fout," ? ");

   (refine.pdbx_overall_ESU_R >ini)? fprintf(fout," %.4f ", refine.pdbx_overall_ESU_R):fprintf(fout," ? ");

   (refine.pdbx_overall_ESU_R_Free>ini)? fprintf(fout,"  %.4f ", refine.pdbx_overall_ESU_R_Free):fprintf(fout," ? ");

   (refine.overall_SU_ML >ini)? fprintf(fout,"  %.4f ", refine.overall_SU_ML):fprintf(fout," ? ");

   (refine.overall_SU_B >ini)? fprintf(fout,"  %.4f ", refine.overall_SU_B):fprintf(fout," ? ");

   (refine.solvent_model_details[0] !='?')? fprintf(fout," \"%s\" ", refine.solvent_model_details):fprintf(fout," ? ");

   (refine.pdbx_solvent_vdw_probe_radii >ini)?fprintf(fout," %.4f ", refine.pdbx_solvent_vdw_probe_radii):fprintf(fout," ? ");

   (refine.pdbx_solvent_ion_probe_radii >ini)? fprintf(fout,"  %.4f ", refine.pdbx_solvent_ion_probe_radii):fprintf(fout," ? ");

   (refine.pdbx_solvent_shrinkage_radii >ini)? fprintf(fout," %.4f ", refine.pdbx_solvent_shrinkage_radii):fprintf(fout," ? ");

   (refine.ls_number_parameters >ini)?  fprintf(fout," %d ", refine.ls_number_parameters):fprintf(fout," ? ");
   
   (refine.ls_number_restraints >ini)? fprintf(fout,"  %d ", refine.ls_number_restraints):fprintf(fout," ? ");
   
   (refine.pdbx_starting_model[0] !='?') ?fprintf(fout," \"%s\" ", refine.pdbx_starting_model):fprintf(fout," ? ");

   if(strstr_case(refine.pdbx_method_to_determine_struct, "mr"))
       strcpy(refine.pdbx_method_to_determine_struct, "MOLECULAR REPLACEMENT");
   
   (refine.pdbx_method_to_determine_struct[0] !='?' && strlen(refine.pdbx_method_to_determine_struct)>0)?
       fprintf(fout," \"%s\" ",refine.pdbx_method_to_determine_struct ):fprintf(fout," ? ");      
   
   (refine.pdbx_stereochemistry_target_values [0] !='?')? fprintf(fout,"  \"%s\"  ", refine.pdbx_stereochemistry_target_values ):fprintf(fout," ? ");

   (refine.pdbx_stereochem_target_val_spec_case[0] !='?')? fprintf(fout,"  \"%s\"  ", refine.pdbx_stereochem_target_val_spec_case):fprintf(fout," ? ");
   

   (refine.overall_FOM_work_R_set >ini)? fprintf(fout," %.4f ", refine.overall_FOM_work_R_set):fprintf(fout," ? ");

   (refine.B_iso_max >ini)?  fprintf(fout," %.3f ", refine.B_iso_max):fprintf(fout," ? ");

   (refine.B_iso_min >ini)?  fprintf(fout," %.3f ", refine.B_iso_min):fprintf(fout," ? ");

   (refine.pdbx_overall_phase_error >ini)? fprintf(fout," %.4f ", refine.pdbx_overall_phase_error):fprintf(fout," ? ");

   (refine.occupancy_max >ini)? fprintf(fout," %.3f ", refine.occupancy_max):fprintf(fout," ? ");

   (refine.occupancy_min >ini)?  fprintf(fout," %.3f ", refine.occupancy_min):fprintf(fout," ? ");


   fprintf(fout,"\n");

   if(refine_neutron.ls_d_res_high>0){
       
    fprintf(fout,"%s  '%s' ",  DATA_BLOCK, REFINE_ID[1] );
    
    (refine_neutron.ls_d_res_high >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_d_res_high):fprintf(fout," ? ");

    (refine_neutron.ls_d_res_low >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_d_res_low):fprintf(fout," ? ");
   
   (refine_neutron.pdbx_ls_sigma_F  >ini)? fprintf(fout,"   %.3f ", refine_neutron.pdbx_ls_sigma_F):fprintf(fout," ? ");
   
   (refine_neutron.pdbx_data_cutoff_high_absF >ini)? fprintf(fout,"  %.4f ", refine_neutron.pdbx_data_cutoff_high_absF):fprintf(fout," ? ");
   
   (refine_neutron.pdbx_data_cutoff_low_absF >ini)? fprintf(fout,"  %.4f ", refine_neutron.pdbx_data_cutoff_low_absF):fprintf(fout," ? ");
   
   (refine_neutron.ls_percent_reflns_obs >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_percent_reflns_obs):fprintf(fout," ? ");
   
   (refine_neutron.ls_number_reflns_obs >ini)? fprintf(fout,"  %d ", refine_neutron.ls_number_reflns_obs):fprintf(fout," ? ");

   (refine_neutron.ls_number_reflns_all >ini)? fprintf(fout,"  %d ", refine_neutron.ls_number_reflns_all):fprintf(fout," ? ");
   
   (refine_neutron.pdbx_ls_cross_valid_method[0] != '?')? fprintf(fout," \"%s\"  ", refine_neutron.pdbx_ls_cross_valid_method):fprintf(fout," ? ");
   
   (refine_neutron.ls_matrix_type[0] != '?') ?fprintf(fout," \"%s\"   ", refine_neutron.ls_matrix_type):fprintf(fout," ? ");
   
   (refine_neutron.pdbx_R_Free_selection_details[0] !='?') ?fprintf(fout," \"%s\"   ", refine_neutron.pdbx_R_Free_selection_details):fprintf(fout," ? ");
 
   (refine_neutron.details[0] !='?' && strlen(refine_neutron.details) >2)? fprintf(fout," \"%s\"  ", refine_neutron.details):fprintf(fout," ? ");
   
   (refine_neutron.ls_R_factor_all >ini)?  fprintf(fout,"  %.4f ", refine_neutron.ls_R_factor_all):fprintf(fout," ? ");
   
   (refine_neutron.ls_R_factor_obs >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_R_factor_obs):fprintf(fout," ? ");
   
   (refine_neutron.ls_R_factor_R_work >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_R_factor_R_work):fprintf(fout," ? ");
   
   (refine_neutron.ls_wR_factor_R_work >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_wR_factor_R_work):fprintf(fout," ? ");
   
   (refine_neutron.ls_R_factor_R_free >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_R_factor_R_free):fprintf(fout," ? ");
   
   (refine_neutron.ls_wR_factor_R_free >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_wR_factor_R_free):fprintf(fout," ? ");
   
   (refine_neutron.ls_percent_reflns_R_free >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_percent_reflns_R_free):fprintf(fout," ? ");
   
   (refine_neutron.ls_number_reflns_R_free >ini)? fprintf(fout,"  %d ", refine_neutron.ls_number_reflns_R_free):fprintf(fout," ? ");

   (refine_neutron.ls_number_reflns_R_work >0)? fprintf(fout,"  %d ", refine_neutron.ls_number_reflns_R_work):fprintf(fout," ? ");
   
   (refine_neutron.ls_R_factor_R_free_error >ini)? fprintf(fout,"  %.4f ", refine_neutron.ls_R_factor_R_free_error):fprintf(fout," ? ");

   (refine_neutron.B_iso_mean >ini)?  fprintf(fout,"  %.4f ", refine_neutron.B_iso_mean):fprintf(fout," ? ");

   (refine_neutron.solvent_model_param_bsol >ini)? fprintf(fout,"  %.4f ", refine_neutron.solvent_model_param_bsol):fprintf(fout," ? ");

   (refine_neutron.solvent_model_param_ksol >ini)? fprintf(fout," %.4f ", refine_neutron.solvent_model_param_ksol):fprintf(fout," ? ");

   (refine_neutron.pdbx_isotropic_thermal_model[0] != '?' )? fprintf(fout," \"%s\"  ", refine_neutron.pdbx_isotropic_thermal_model):fprintf(fout," ? ");

   (refine_neutron.aniso_B11 >ini)? fprintf(fout,"  %.4f ", refine_neutron.aniso_B11):fprintf(fout," ? ");

   (refine_neutron.aniso_B22 >ini)? fprintf(fout,"  %.4f ", refine_neutron.aniso_B22):fprintf(fout," ? ");

   (refine_neutron.aniso_B33 >ini)?  fprintf(fout,"  %.4f ", refine_neutron.aniso_B33):fprintf(fout," ? ");

   (refine_neutron.aniso_B12 >ini)? fprintf(fout,"  %.4f ", refine_neutron.aniso_B12):fprintf(fout," ? ");

   (refine_neutron.aniso_B13 >ini)? fprintf(fout," %.4f ", refine_neutron.aniso_B13):fprintf(fout," ? ");

   (refine_neutron.aniso_B23 >ini)? fprintf(fout,"  %.4f ", refine_neutron.aniso_B23):fprintf(fout," ? ");

   (refine_neutron.correlation_coeff_Fo_to_Fc >ini)? fprintf(fout,"  %.4f ", refine_neutron.correlation_coeff_Fo_to_Fc):fprintf(fout," ? ");

   (refine_neutron.correlation_coeff_Fo_to_Fc_free >ini)?fprintf(fout,"  %.4f ", refine_neutron.correlation_coeff_Fo_to_Fc_free):fprintf(fout," ? ");

   (refine_neutron.overall_SU_R_Cruickshank_DPI >ini)? fprintf(fout,"  %.4f ", refine_neutron.overall_SU_R_Cruickshank_DPI):fprintf(fout," ? ");

   (refine_neutron.overall_SU_R_free >ini)? fprintf(fout,"  %.4f ", refine_neutron.overall_SU_R_free):fprintf(fout," ? ");

   (refine_neutron.pdbx_overall_ESU_R >ini)? fprintf(fout," %.4f ", refine_neutron.pdbx_overall_ESU_R):fprintf(fout," ? ");

   (refine_neutron.pdbx_overall_ESU_R_Free>ini)? fprintf(fout,"  %.4f ", refine_neutron.pdbx_overall_ESU_R_Free):fprintf(fout," ? ");

   (refine_neutron.overall_SU_ML >ini)? fprintf(fout,"  %.4f ", refine_neutron.overall_SU_ML):fprintf(fout," ? ");

   (refine_neutron.overall_SU_B >ini)? fprintf(fout,"  %.4f ", refine_neutron.overall_SU_B):fprintf(fout," ? ");

   (refine_neutron.solvent_model_details[0] !='?')? fprintf(fout," \"%s\" ", refine_neutron.solvent_model_details):fprintf(fout," ? ");

   (refine_neutron.pdbx_solvent_vdw_probe_radii >ini)?fprintf(fout," %.4f ", refine_neutron.pdbx_solvent_vdw_probe_radii):fprintf(fout," ? ");

   (refine_neutron.pdbx_solvent_ion_probe_radii >ini)? fprintf(fout,"  %.4f ", refine_neutron.pdbx_solvent_ion_probe_radii):fprintf(fout," ? ");

   (refine_neutron.pdbx_solvent_shrinkage_radii >ini)? fprintf(fout," %.4f ", refine_neutron.pdbx_solvent_shrinkage_radii):fprintf(fout," ? ");

   (refine_neutron.ls_number_parameters >ini)?  fprintf(fout," %d ", refine_neutron.ls_number_parameters):fprintf(fout," ? ");
   
   (refine_neutron.ls_number_restraints >ini)? fprintf(fout,"  %d ", refine_neutron.ls_number_restraints):fprintf(fout," ? ");
   
   (refine_neutron.pdbx_starting_model[0] !='?') ?fprintf(fout," \"%s\" ", refine_neutron.pdbx_starting_model):fprintf(fout," ? ");
   
   (refine_neutron.pdbx_method_to_determine_struct[0] !='?' && strlen(refine_neutron.pdbx_method_to_determine_struct)>0)?
       fprintf(fout," \"%s\" ",refine_neutron.pdbx_method_to_determine_struct ):fprintf(fout," ? ");      
   
   (refine_neutron.pdbx_stereochemistry_target_values [0] !='?')? fprintf(fout,"  \"%s\"  ", refine_neutron.pdbx_stereochemistry_target_values ):fprintf(fout," ? ");

   (refine_neutron.pdbx_stereochem_target_val_spec_case[0] !='?')? fprintf(fout,"  \"%s\"  ", refine_neutron.pdbx_stereochem_target_val_spec_case):fprintf(fout," ? ");
   

   (refine_neutron.overall_FOM_work_R_set >ini)? fprintf(fout," %.4f ", refine_neutron.overall_FOM_work_R_set):fprintf(fout," ? ");

   (refine_neutron.B_iso_max >ini)?  fprintf(fout," %.3f ", refine_neutron.B_iso_max):fprintf(fout," ? ");

   (refine_neutron.B_iso_min >ini)?  fprintf(fout," %.3f ", refine_neutron.B_iso_min):fprintf(fout," ? ");

   (refine_neutron.pdbx_overall_phase_error >ini)? fprintf(fout," %.4f ", refine_neutron.pdbx_overall_phase_error):fprintf(fout," ? ");

   (refine_neutron.occupancy_max >ini)? fprintf(fout," %.3f ", refine_neutron.occupancy_max):fprintf(fout," ? ");

   (refine_neutron.occupancy_min >ini)?  fprintf(fout," %.3f ", refine_neutron.occupancy_min):fprintf(fout," ? ");


fprintf(fout,"\n");
 
   }
   


 

   
   fprintf(fout,"#\n");  //for pdbx_refine
   if(pdbx_refine.R_factor_all_4sig_cutoff >INI){
       fprintf(fout,"_pdbx_refine.pdbx_refine_id  '%s' \n", REFINE_ID[0]);
       fprintf(fout,"_pdbx_refine.R_factor_all_4sig_cutoff %.4f\n", pdbx_refine.R_factor_all_4sig_cutoff); 
   }
   
   if(pdbx_refine.R_factor_all_no_cutoff >INI)
       fprintf(fout,"_pdbx_refine.R_factor_all_no_cutoff %.4f\n", pdbx_refine.R_factor_all_no_cutoff); 
   if(pdbx_refine.R_factor_obs_4sig_cutoff >INI)
       fprintf(fout,"_pdbx_refine.R_factor_obs_4sig_cutoff %.4f\n", pdbx_refine.R_factor_obs_4sig_cutoff); 
   if(pdbx_refine.R_factor_obs_no_cutoff >INI)
       fprintf(fout,"_pdbx_refine.R_factor_obs_no_cutoff %.4f\n", pdbx_refine.R_factor_obs_no_cutoff); 
   if(pdbx_refine.free_R_factor_4sig_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_factor_4sig_cutoff %.4f\n", pdbx_refine.free_R_factor_4sig_cutoff); 
   if(pdbx_refine.free_R_factor_no_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_factor_no_cutoff %.4f\n", pdbx_refine.free_R_factor_no_cutoff); 
   if(pdbx_refine.free_R_val_4sig_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_val_4sig_cutoff %.4f\n", pdbx_refine.free_R_val_4sig_cutoff); 
   if(pdbx_refine.free_R_val_test_set_ct_4sig_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_val_test_set_ct_4sig_cutoff %d\n", pdbx_refine.free_R_val_test_set_ct_4sig_cutoff); 
   if(pdbx_refine.free_R_val_test_set_ct_no_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_val_test_set_ct_no_cutoff %d\n", pdbx_refine.free_R_val_test_set_ct_no_cutoff); 
   if(pdbx_refine.free_R_val_test_set_size_perc_4sig_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_val_test_set_size_perc_4sig_cutoff %.4f\n", pdbx_refine.free_R_val_test_set_size_perc_4sig_cutoff); 
   if(pdbx_refine.free_R_val_test_set_size_perc_no_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_val_test_set_size_perc_no_cutoff %.4f\n", pdbx_refine.free_R_val_test_set_size_perc_no_cutoff); 
   if(pdbx_refine.number_reflns_obs_4sig_cutoff >INI)
       fprintf(fout,"_pdbx_refine.number_reflns_obs_4sig_cutoff %d\n", pdbx_refine.number_reflns_obs_4sig_cutoff); 
   if(pdbx_refine.number_reflns_obs_no_cutoff >INI)
       fprintf(fout,"_pdbx_refine.number_reflns_obs_no_cutoff %d\n", pdbx_refine.number_reflns_obs_no_cutoff); 
   if(pdbx_refine.free_R_error_no_cutoff >INI)
       fprintf(fout,"_pdbx_refine.free_R_error_no_cutoff %.4f\n", pdbx_refine.free_R_error_no_cutoff); 

}


void write_refine_ls_shell(FILE *fout)
{
    int m;

    if(refine_ls_shell.nbin<=0) return;
       
    fprintf(fout,"#\n");
    fprintf(fout,"loop_\n");
    fprintf(fout,"_refine_ls_shell.d_res_high\n");
    fprintf(fout,"_refine_ls_shell.d_res_low\n");
    fprintf(fout,"_refine_ls_shell.pdbx_total_number_of_bins_used\n");
    fprintf(fout,"_refine_ls_shell.percent_reflns_obs\n");
    fprintf(fout,"_refine_ls_shell.number_reflns_R_work\n");
    fprintf(fout,"_refine_ls_shell.R_factor_all\n");
    fprintf(fout,"_refine_ls_shell.R_factor_R_work\n");
    fprintf(fout,"_refine_ls_shell.R_factor_R_free\n");
    fprintf(fout,"_refine_ls_shell.percent_reflns_R_free\n");
    fprintf(fout,"_refine_ls_shell.number_reflns_R_free\n");
    fprintf(fout,"_refine_ls_shell.R_factor_R_free_error\n");
    fprintf(fout,"_refine_ls_shell.number_reflns_all\n");
    fprintf(fout,"_refine_ls_shell.number_reflns_obs\n");
    fprintf(fout,"_refine_ls_shell.pdbx_refine_id\n");
    
    for(m=0; m<refine_ls_shell.nbin; m++){
    
        (refine_ls_shell.d_res_high[m] >-9998) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.d_res_high[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.d_res_low[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.d_res_low[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.pdbx_total_number_of_bins_used[m] > 0) ?
            fprintf(fout,"%d  ", refine_ls_shell.pdbx_total_number_of_bins_used[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.percent_reflns_obs[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.percent_reflns_obs[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.number_reflns_R_work[m] > 0) ?
            fprintf(fout,"%d  ", refine_ls_shell.number_reflns_R_work[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.R_factor_all[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.R_factor_all[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.R_factor_R_work[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.R_factor_R_work[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.R_factor_R_free[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.R_factor_R_free[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.percent_reflns_R_free[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.percent_reflns_R_free[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.number_reflns_R_free[m] > 0) ?
            fprintf(fout,"%d  ", refine_ls_shell.number_reflns_R_free[m]) : fprintf(fout,"?  ");
    
        (refine_ls_shell.R_factor_R_free_error[m] > 0  ) ?
            fprintf(fout,"%.4f  ", refine_ls_shell.R_factor_R_free_error[m]) : fprintf(fout,"?  ");


        if(refine_ls_shell.number_reflns_R_work[m]>0 && refine_ls_shell.number_reflns_R_free[m]>0
           && refine_ls_shell.number_reflns_all[m]<=0){
            refine_ls_shell.number_reflns_all[m] = refine_ls_shell.number_reflns_R_work[m] +
                refine_ls_shell.number_reflns_R_free[m];
        }
        
        (refine_ls_shell.number_reflns_all[m] > 0  ) ?
            fprintf(fout,"%d  ", refine_ls_shell.number_reflns_all[m]) : fprintf(fout,"?  ");
        
        (refine_ls_shell.number_reflns_obs[m] > 0  ) ?
            fprintf(fout,"%d  ", refine_ls_shell.number_reflns_obs[m]) : fprintf(fout,"?  ");

            fprintf(fout," '%s' ", REFINE_ID[0]) ;
        
        fprintf(fout,"\n");
        
    }


    for(m=0; m<refine_shell_neutron.nbin; m++){
         (refine_shell_neutron.d_res_high[m] >-9998) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.d_res_high[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.d_res_low[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.d_res_low[m]) : fprintf(fout,"?  ");

        fprintf(fout,"%d  ", refine_shell_neutron.nbin);
        
//        (refine_shell_neutron.pdbx_total_number_of_bins_used[m] > 0) ?            fprintf(fout,"%d  ", refine_shell_neutron.pdbx_total_number_of_bins_used[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.percent_reflns_obs[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.percent_reflns_obs[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.number_reflns_R_work[m] > 0) ?
            fprintf(fout,"%d  ", refine_shell_neutron.number_reflns_R_work[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.R_factor_all[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.R_factor_all[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.R_factor_R_work[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.R_factor_R_work[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.R_factor_R_free[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.R_factor_R_free[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.percent_reflns_R_free[m] > 0) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.percent_reflns_R_free[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.number_reflns_R_free[m] > 0) ?
            fprintf(fout,"%d  ", refine_shell_neutron.number_reflns_R_free[m]) : fprintf(fout,"?  ");
    
        (refine_shell_neutron.R_factor_R_free_error[m] > 0  ) ?
            fprintf(fout,"%.4f  ", refine_shell_neutron.R_factor_R_free_error[m]) : fprintf(fout,"?  ");


        if(refine_shell_neutron.number_reflns_R_work[m]>0 && refine_shell_neutron.number_reflns_R_free[m]>0
           && refine_shell_neutron.number_reflns_all[m]<=0){
            refine_shell_neutron.number_reflns_all[m] = refine_shell_neutron.number_reflns_R_work[m] +
                refine_shell_neutron.number_reflns_R_free[m];
        }
        
        (refine_shell_neutron.number_reflns_all[m] > 0  ) ?
            fprintf(fout,"%d  ", refine_shell_neutron.number_reflns_all[m]) : fprintf(fout,"?  ");
        
        (refine_shell_neutron.number_reflns_obs[m] > 0  ) ?
            fprintf(fout,"%d  ", refine_shell_neutron.number_reflns_obs[m]) : fprintf(fout,"?  ");

            fprintf(fout," '%s'  ", REFINE_ID[1]) ;
        
        fprintf(fout,"\n");
        
    }
   
}

/*
void write_pdbx_refine_ls_shell_neutron(FILE *fout)
{
    int m;

    if(pdbx_refine_ls_shell_neutron.nbin<=0) return;
       
    fprintf(fout,"#\n");
    fprintf(fout,"loop_\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.d_res_high\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.d_res_low\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.total_number_of_bins_used\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.percent_reflns_obs\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.number_reflns_R_work\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.R_factor_all\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.R_factor_R_work\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.R_factor_R_free\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.percent_reflns_R_free\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.number_reflns_R_free\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.R_factor_R_free_error\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.number_reflns_all\n");
    fprintf(fout,"_pdbx_refine_ls_shell_neutron.number_reflns_obs\n");
    
    for(m=0; m<pdbx_refine_ls_shell_neutron.nbin; m++){
    
        (pdbx_refine_ls_shell_neutron.d_res_high[m] >-9998) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.d_res_high[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.d_res_low[m] > 0) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.d_res_low[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.pdbx_total_number_of_bins_used[m] > 0) ?
            fprintf(fout,"%d  ", pdbx_refine_ls_shell_neutron.pdbx_total_number_of_bins_used[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.percent_reflns_obs[m] > 0) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.percent_reflns_obs[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.number_reflns_R_work[m] > 0) ?
            fprintf(fout,"%d  ", pdbx_refine_ls_shell_neutron.number_reflns_R_work[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.R_factor_all[m] > 0) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.R_factor_all[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.R_factor_R_work[m] > 0) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.R_factor_R_work[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.R_factor_R_free[m] > 0) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.R_factor_R_free[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.percent_reflns_R_free[m] > 0) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.percent_reflns_R_free[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.number_reflns_R_free[m] > 0) ?
            fprintf(fout,"%d  ", pdbx_refine_ls_shell_neutron.number_reflns_R_free[m]) : fprintf(fout,"?  ");
    
        (pdbx_refine_ls_shell_neutron.R_factor_R_free_error[m] > 0  ) ?
            fprintf(fout,"%.3f  ", pdbx_refine_ls_shell_neutron.R_factor_R_free_error[m]) : fprintf(fout,"?  ");


        if(pdbx_refine_ls_shell_neutron.number_reflns_R_work[m]>0 && pdbx_refine_ls_shell_neutron.number_reflns_R_free[m]>0
           && pdbx_refine_ls_shell_neutron.number_reflns_all[m]<=0){
            pdbx_refine_ls_shell_neutron.number_reflns_all[m] = pdbx_refine_ls_shell_neutron.number_reflns_R_work[m] +
                pdbx_refine_ls_shell_neutron.number_reflns_R_free[m];
        }
        
        (pdbx_refine_ls_shell_neutron.number_reflns_all[m] > 0  ) ?
            fprintf(fout,"%d  ", pdbx_refine_ls_shell_neutron.number_reflns_all[m]) : fprintf(fout,"?  ");
        
        (pdbx_refine_ls_shell_neutron.number_reflns_obs[m] > 0  ) ?
            fprintf(fout,"%d  ", pdbx_refine_ls_shell_neutron.number_reflns_obs[m]) : fprintf(fout,"?  ");

        
        fprintf(fout,"\n");
        
    }
    
}
*/

void write_reflns_shell(FILE *fout)
{
    int m;

    if(reflns_shell.nbin<=0) return;
        
    fprintf(fout, "\n# statistics from LOG files (_reflns.shells)\n");
    fprintf(fout, "\nloop_\n");
    
    fprintf(fout, "_reflns_shell.d_res_high\n");
    fprintf(fout, "_reflns_shell.d_res_low\n");

    fprintf(fout, "_reflns_shell.number_measured_obs\n");
    fprintf(fout, "_reflns_shell.number_measured_all\n");
    fprintf(fout, "_reflns_shell.number_unique_obs \n");
    fprintf(fout, "_reflns_shell.pdbx_rejects\n");

    fprintf(fout, "_reflns_shell.Rmerge_I_obs\n");
    fprintf(fout, "_reflns_shell.meanI_over_sigI_obs \n");
    
    fprintf(fout, "_reflns_shell.pdbx_Rsym_value \n");
    fprintf(fout, "_reflns_shell.pdbx_chi_squared\n");
    fprintf(fout, "_reflns_shell.pdbx_redundancy\n");
    fprintf(fout, "_reflns_shell.percent_possible_obs\n");

    fprintf(fout, "_reflns_shell.pdbx_Rmeas_mean \n");
//    fprintf(fout, "_reflns_shell.pdbx_netI_over_sigmaI_obs \n");
    fprintf(fout, "_reflns_shell.pdbx_netI_over_sigmaI_obs \n");
    
    fprintf(fout, "_reflns_shell.pdbx_number_centric \n");
    fprintf(fout, "_reflns_shell.pdbx_number_anomalous \n");
    fprintf(fout, "_reflns_shell.pdbx_Rmerge_I_anomalous \n");
    fprintf(fout, "_reflns_shell.pdbx_meanI_over_sigI_anomalous \n");
    fprintf(fout, "_reflns_shell.pdbx_PCV_mean \n");
    
        
    fprintf(fout, "_reflns_shell.number_possible\n");
    fprintf(fout, "_reflns_shell.number_unique_all\n");
   
    fprintf(fout, "_reflns_shell.Rmerge_F_all\n");
    fprintf(fout, "_reflns_shell.Rmerge_F_obs\n");
    fprintf(fout, "_reflns_shell.Rmerge_I_all\n");
    
    fprintf(fout, "_reflns_shell.meanI_over_sigI_all\n");
    fprintf(fout, "_reflns_shell.percent_possible_all\n");

    fprintf(fout, "_reflns_shell.pdbx_Rrim_I_all\n");
    fprintf(fout, "_reflns_shell.pdbx_Rpim_I_all\n");
    
    for(m=0; m<reflns_shell.nbin; m++){
        
        (reflns_shell.d_res_high[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.d_res_high[m]) : fprintf(fout," ? ");
        
        (reflns_shell.d_res_low[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.d_res_low[m]) : fprintf(fout," ? ");


        (reflns_shell.number_measured_obs[m]>0) ?
            fprintf(fout," %d ", reflns_shell.number_measured_obs[m]): fprintf(fout," ? ");

        (reflns_shell.number_measured_all[m]>0) ?
            fprintf(fout," %d ", reflns_shell.number_measured_all[m]): fprintf(fout," ? ");
        
        (reflns_shell.number_unique_obs[m]>0) ?
            fprintf(fout," %d ",reflns_shell.number_unique_obs[m]): fprintf(fout," ? ");

        (reflns_shell.pdbx_rejects[m]>0) ?
            fprintf(fout," %d ", reflns_shell.pdbx_rejects[m]): fprintf(fout," ? ");


        (reflns_shell.Rmerge_I_obs[m]>0) ?
            fprintf(fout," %.3f ",reflns_shell.Rmerge_I_obs[m]): fprintf(fout," ? ");



        (reflns_shell.meanI_over_sigI_obs[m]>0) ?
            fprintf(fout," %.1f ", reflns_shell.meanI_over_sigI_obs[m]): fprintf(fout," ? ");
        
        (reflns_shell.pdbx_Rsym_value[m]>0) ?
            fprintf(fout," %.3f \n",reflns_shell.pdbx_Rsym_value[m]):fprintf(fout," ? ");
        (reflns_shell.pdbx_chi_squared[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.pdbx_chi_squared[m]): fprintf(fout," ? ");
        (reflns_shell.pdbx_redundancy[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.pdbx_redundancy[m]): fprintf(fout," ? ");
        
        (reflns_shell.percent_possible_obs[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.percent_possible_obs[m]): fprintf(fout," ? ");


        (reflns_shell.pdbx_Rmeas_mean[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.pdbx_Rmeas_mean[m]): fprintf(fout," ? ");

        (reflns_shell.pdbx_netI_over_sigmaI_obs[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.pdbx_netI_over_sigmaI_obs[m]): fprintf(fout," ? ");


        
        (reflns_shell.pdbx_number_centric[m]>0) ?
            fprintf(fout," %d ", reflns_shell.pdbx_number_centric[m]): fprintf(fout," ? ");

        (reflns_shell.pdbx_number_anomalous[m]>0) ?
            fprintf(fout," %d ",reflns_shell.pdbx_number_anomalous[m] ): fprintf(fout," ? ");

        (reflns_shell.pdbx_Rmerge_I_anomalous[m]>0) ?
            fprintf(fout," %.3f ",reflns_shell.pdbx_Rmerge_I_anomalous[m] ): fprintf(fout," ? ");

        (reflns_shell.pdbx_meanI_over_sigI_anomalous[m]>0) ?
            fprintf(fout," %.1f ",reflns_shell.pdbx_meanI_over_sigI_anomalous[m] ): fprintf(fout," ? ");

        (reflns_shell.pdbx_PCV_mean[m]>0) ?
            fprintf(fout," %.3f ",reflns_shell.pdbx_PCV_mean[m] ): fprintf(fout," ? ");

        

        
        (reflns_shell.number_possible[m]>0) ?
            fprintf(fout," %d ",reflns_shell.number_possible[m]): fprintf(fout," ? ");
        
        (reflns_shell.number_unique_all[m]>0) ?
            fprintf(fout," %d ", reflns_shell.number_unique_all[m]): fprintf(fout," ? ");


        
        (reflns_shell.Rmerge_F_all[m]>0) ?
            fprintf(fout," %.3f ",reflns_shell.Rmerge_F_all[m]): fprintf(fout," ? ");
        
        (reflns_shell.Rmerge_F_obs[m]>0) ?
            fprintf(fout," %.3f ",reflns_shell.Rmerge_F_obs[m]): fprintf(fout," ? ");
        
        (reflns_shell.Rmerge_I_all[m]>0) ?
            fprintf(fout," %.3f ",reflns_shell.Rmerge_I_all[m]): fprintf(fout," ? ");

        (reflns_shell.meanI_over_sigI_all[m]>0) ?
            fprintf(fout," %.1f ", reflns_shell.meanI_over_sigI_all[m]): fprintf(fout," ? ");
        

        (reflns_shell.percent_possible_all[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.percent_possible_all[m]): fprintf(fout," ? ");

        (reflns_shell.pdbx_Rrim_I_all[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.pdbx_Rrim_I_all[m]): fprintf(fout," ? ");
        (reflns_shell.pdbx_Rpim_I_all[m]>0) ?
            fprintf(fout," %.3f ", reflns_shell.pdbx_Rpim_I_all[m]): fprintf(fout," ? ");

        fprintf(fout," \n");
        
    }
    
}

void write_reflns(FILE *fout)
{
    fprintf(fout, "\n# statistics from LOG files (_reflns)\n");


    if(reflns.pdbx_average_I_obs>0 &&  reflns.pdbx_average_sigmaI_obs>0)
        reflns.pdbx_netI_over_av_sigmaI= reflns.pdbx_average_I_obs/reflns.pdbx_average_sigmaI_obs;
   

    fprintf(fout, "_reflns.entry_id  '%s' \n",  DATA_BLOCK);

   
    if(reflns.d_resolution_high>0)
        fprintf(fout, "_reflns.d_resolution_high  %.3f\n", reflns.d_resolution_high);
    
    if(reflns.d_resolution_low>0)
        fprintf(fout, "_reflns.d_resolution_low  %.3f\n", reflns.d_resolution_low);

    if(reflns.pdbx_number_measured_all>0)
        fprintf(fout, "_reflns.pdbx_number_measured_all %d\n", reflns.pdbx_number_measured_all);
    
    if(reflns.number_all>0)
        fprintf(fout, "_reflns.number_all  %d \n", reflns.number_all);
    
    if(reflns.number_obs>0)
        fprintf(fout, "_reflns.number_obs  %d\n", reflns.number_obs);
    
    if(reflns.pdbx_scaling_rejects>0)
        fprintf(fout, "_reflns.pdbx_scaling_rejects  %d\n", reflns.pdbx_scaling_rejects);
    

    if(reflns.pdbx_number_centric>0)
        fprintf(fout, "_reflns.pdbx_number_centric %d\n", reflns.pdbx_number_centric);
    
    if(reflns.pdbx_number_anomalous>0)
        fprintf(fout, "_reflns.pdbx_number_anomalous %d\n", reflns.pdbx_number_anomalous);


    if(reflns.pdbx_Rmerge_I_obs>0)
        fprintf(fout, "_reflns.pdbx_Rmerge_I_obs  %.3f\n", reflns.pdbx_Rmerge_I_obs);

    if(reflns.pdbx_netI_over_av_sigmaI>0)
        fprintf(fout, "_reflns.pdbx_netI_over_av_sigmaI  %.3f\n", reflns.pdbx_netI_over_av_sigmaI);

    if(reflns.pdbx_netI_over_sigmaI>0)
        fprintf(fout, "_reflns.pdbx_netI_over_sigmaI  %.3f\n", reflns.pdbx_netI_over_sigmaI);

    
    if(reflns.pdbx_Rsym_value>0)
        fprintf(fout, "_reflns.pdbx_Rsym_value  %.3f\n", reflns.pdbx_Rsym_value);
    
    if(reflns.pdbx_chi_squared>0)
        fprintf(fout, "_reflns.pdbx_chi_squared  %.3f\n", reflns.pdbx_chi_squared);

    if(reflns.pdbx_redundancy>0)
        fprintf(fout, "_reflns.pdbx_redundancy  %.3f\n", reflns.pdbx_redundancy);
    
    if(reflns.percent_possible_obs>0)
        fprintf(fout, "_reflns.percent_possible_obs %.3f\n", reflns.percent_possible_obs);
    
    if(reflns.pdbx_Rmeas_mean>0)
        fprintf(fout, "_reflns.pdbx_Rmeas_mean %.3f\n", reflns.pdbx_Rmeas_mean);

    if(reflns.pdbx_netI_over_sigmaI>0)
        fprintf(fout, "_reflns.pdbx_netI_over_sigmaI %.3f\n", reflns.pdbx_netI_over_sigmaI);
    
    if(reflns.R_free_details[0]!='?')
        fprintf(fout, "_reflns.R_free_details  '%s' \n", reflns.R_free_details);
    if(reflns.details[0]!='?')
        fprintf(fout, "_reflns.details  '%s' \n", reflns.details);
    if(reflns.observed_criterion[0]!='?')
        fprintf(fout, "_reflns.observed_criterion  '%s' \n", reflns.observed_criterion);
    
    if(reflns.B_iso_Wilson_estimate>0)
        fprintf(fout, "_reflns.B_iso_Wilson_estimate  %.3f\n", reflns.B_iso_Wilson_estimate);
    
    if(reflns.Rmerge_F_all>0)
        fprintf(fout, "_reflns.Rmerge_F_all  %.3f\n", reflns.Rmerge_F_all);
    
    if(reflns.Rmerge_F_obs>0)
        fprintf(fout, "_reflns.Rmerge_F_obs  %.3f\n", reflns.Rmerge_F_obs);
    
    
    if(reflns.limit_h_max>INI)
        fprintf(fout, "_reflns.limit_h_max  %d\n",reflns.limit_h_max);
    
    if(reflns.limit_h_min>INI)
        fprintf(fout, "_reflns.limit_h_min  %d\n",reflns.limit_h_min);

    if(reflns.limit_k_max>INI)
        fprintf(fout, "_reflns.limit_k_max  %d\n",reflns.limit_k_max);
    
    if(reflns.limit_k_min>INI)
        fprintf(fout, "_reflns.limit_k_min  %d\n",reflns.limit_k_min);

    if(reflns.limit_l_max>INI)
        fprintf(fout, "_reflns.limit_l_max  %d\n",reflns.limit_l_max);
    
    if(reflns.limit_l_min>INI)
        fprintf(fout, "_reflns.limit_l_min  %d\n",reflns.limit_l_min);

    
    if(reflns.observed_criterion_F_max>INI)
        fprintf(fout, "_reflns.observed_criterion_F_max  %.3f\n",reflns.observed_criterion_F_max );

    if(reflns.observed_criterion_F_min>INI)
        fprintf(fout, "_reflns.observed_criterion_F_min  %.3f\n",reflns.observed_criterion_F_min );

    if(reflns.observed_criterion_I_max>INI)
        fprintf(fout, "_reflns.observed_criterion_I_max  %.3f\n",reflns.observed_criterion_I_max );

    if(reflns.observed_criterion_I_min>INI)
        fprintf(fout, "_reflns.observed_criterion_I_min  %.3f\n",reflns.observed_criterion_I_min );

    if(reflns.observed_criterion_sigma_F>INI)
        fprintf(fout, "_reflns.observed_criterion_sigma_F  %.3f\n",reflns.observed_criterion_sigma_F);
    
    if(reflns.observed_criterion_sigma_I>INI)
        fprintf(fout, "_reflns.observed_criterion_sigma_I  %.3f\n",reflns.observed_criterion_sigma_I);

 
 
    
    
    if(reflns.pdbx_d_res_high_opt>0)
        fprintf(fout, "_reflns.pdbx_d_res_high_opt  %.3f\n", reflns.pdbx_d_res_high_opt);
    
    if(reflns.pdbx_d_res_low_opt>0)
        fprintf(fout, "_reflns.pdbx_d_res_low_opt %.3f\n", reflns.pdbx_d_res_low_opt);
    



    if(reflns.pdbx_Rmerge_F_obs>0)
        fprintf(fout, "_reflns.pdbx_Rmerge_F_obs %.3f\n", reflns.pdbx_Rmerge_F_obs);

/*    
    if(reflns.pdbx_resolution_low_scale>0)
        fprintf(fout, "_reflns.pdbx_resolution_low_scale %.3f\n", reflns.pdbx_resolution_low_scale);
    
    if(reflns.pdbx_resolution_high_scale>0)
        fprintf(fout, "_reflns.pdbx_resolution_high_scale %.3f\n", reflns.pdbx_resolution_high_scale);
*/

    if(reflns.pdbx_Rmerge_I_anomalous>0)
        fprintf(fout, "_reflns.pdbx_Rmerge_I_anomalous %.3f\n", reflns.pdbx_Rmerge_I_anomalous);
    
    if(reflns.pdbx_netI_over_av_sigmaI_anomalous>0)
        fprintf(fout, "_reflns.pdbx_netI_over_av_sigmaI_anomalous %.3f\n", reflns.pdbx_netI_over_av_sigmaI_anomalous);

    if(reflns.pdbx_PCV_mean>0)
        fprintf(fout, "_reflns.pdbx_PCV_mean %.3f\n", reflns.pdbx_PCV_mean);
    

    
    if(reflns.pdbx_average_I_obs>0)
        fprintf(fout, "_reflns.pdbx_average_I_obs %.3f\n", reflns.pdbx_average_I_obs);

    if(reflns.pdbx_average_sigmaI_obs>0)
        fprintf(fout, "_reflns.pdbx_average_sigmaI_obs %.3f\n", reflns.pdbx_average_sigmaI_obs);
    

    if(reflns.pdbx_Rrim_I_all>0)
        fprintf(fout, "_reflns.pdbx_Rrim_I_all %.3f\n", reflns.pdbx_Rrim_I_all);

    if(reflns.pdbx_Rpim_I_all>0)
        fprintf(fout, "_reflns.pdbx_Rpim_I_all %.3f\n", reflns.pdbx_Rpim_I_all);

    if(reflns.pdbx_d_opt>0)
        fprintf(fout, "_reflns.pdbx_d_opt %.3f\n", reflns.pdbx_d_opt);

}


